/*
    Digital circuits emulation library

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DCE_PROBE_H_
#define DCE_PROBE_H_

#include "dce.h"

class DProbe
{
public:
    DProbe() {
        state_ = 0;
    }

    void connect( unsigned bit, DChip & chip, unsigned line ) {
        lines_[ bit ].connectTo( chip.getLine(line) );
    }

    void connect( unsigned bit, DLine & line ) {
        lines_[ bit ].connectTo( line );
    }

    unsigned update() {
        state_ = 0;

        for( int i=0; i<MaxLines; i++ ) {
            if( lines_[i].state() ) {
                state_ |= 1 << i;
            }
        }

        return state_;
    }

    unsigned state() const {
        return state_;
    }

private:
    enum {
        MaxLines = 8*sizeof(unsigned) // Portable enough...
    };

    DLine lines_[MaxLines];
    unsigned state_;
};

#endif // DCE_PROBE_H_
